/**
 * @file
 * @brief CMSIS RTOS2 queue implementation.
 */
#ifndef STA_RTOS2_QUEUE_HPP
#define STA_RTOS2_QUEUE_HPP

#include <cmsis_os2.h>

#include <cstdint>


namespace sta
{
	/**
	 * @brief Interface object for using CMSIS RTOS2 queues.
	 *
	 * @tparam Message type
	 *
	 * @ingroup rtos2API
	 */
	template <typename T>
	class Rtos2Queue
	{
	public:
		using Message = T;  /**< Queue message type */

	public:
		/**
		 * @param handle CMSIS RTOS2 queue handle
		 */
		Rtos2Queue(osMessageQueueId_t * handle);

		/**
		 * @brief Place message in queue.
		 *
		 * @param msg       Message object
		 * @param timeout   Timeout
		 * @return          True on success
		 */
		bool put(const Message & msg, uint32_t timeout = osWaitForever);
		/**
		 * @brief Take message from queue.
		 *
		 * @param[out] outMsg   Message object destination
		 * @param      timeout  Timeout
		 * @return              True on success
		 */
		bool get(Message * outMsg, uint32_t timeout = osWaitForever);

		/**
		 * @brief Get number of messages in queue.
		 *
		 * @return Number of messages in queue
		 */
		uint32_t available() const;

	private:
		osMessageQueueId_t * handle_;   /**< CMSIS RTOS2 queue handle */
	};
} // namespace sta


#include <sta/rtos2/queue.tpp>


#endif // STA_RTOS2_QUEUE_HPP
